/*
 * XmlReportListener.java
 *  
 * Copyright 2008 C.A. Meijer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jmunit.anttask.cldc10;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;

import jmunit.framework.cldc10.AssertionFailedException;
import jmunit.framework.cldc10.TestListener;

import org.apache.tools.ant.BuildException;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;

/**
 * The XmlReportListener is a JMUnit <code>TestListener</code> that writes
 * test results to an XML file in the format processed by the Ant junitreport
 * task.
 * 
 * This source uses the JDOM API for generating the XML document.
 * 
 * @author C.A. Meijer
 * @since JMUnit 1.1
 */
public class XmlReportListener implements TestListener {

	// The name of the test suite.
	private String testSuiteName;

	// The XML document.
	private Document document;

	// The directory where the document must be saved.
	private String todir;

	// The root element of the document.
	private Element testSuiteElement;

	// The time at which the suite started running.
	private long suiteStartTime;

	// The number of tests in the suite.
	private int numTests;

	// The number of test errors.
	private int numErrors;

	// The number of test failures.
	private int numFailures;

	// The number of completed test cases.
	private int numTestsCompleted;

	// The start time of the current test case.
	private long testcaseStartTime;

	// An failure thrown by the current test case.
	private AssertionFailedException afe;

	// An error reported by the current test case.
	private Throwable throwable;

	/**
	 * Constructor.
	 * 
	 * @param test
	 *            The test.
	 * 
	 * @param todir
	 *            The directory where the report must be written.
	 */
	public XmlReportListener(jmunit.framework.cldc10.Test test, String todir) {
		this.numTests = test.countTestCases();
		this.suiteStartTime = System.currentTimeMillis();
		this.testSuiteElement = new Element("testsuite");
		this.document = new Document(this.testSuiteElement);
		this.testSuiteName = test.getClass().getName();
		this.todir = todir + File.separator;
	}

	/**
	 * A test has started running.
	 * 
	 * @param testClass
	 *            The class of the test.
	 * 
	 * @param testName
	 *            The name of the test case.
	 * 
	 */
	public void startTest(Class testClass, String testName) {
		this.testcaseStartTime = System.currentTimeMillis();
		this.afe = null;
		this.throwable = null;
	}

	/**
	 * Report an error.
	 * 
	 * @param testClass
	 *            The class of the test.
	 * 
	 * @param testName
	 *            The name of the test case.
	 * 
	 * @param t
	 *            The error.
	 */
	public void addError(Class testClass, String testName, Throwable t) {
		this.numErrors++;
		this.throwable = t;
	}

	/**
	 * Report a failure.
	 * 
	 * @param testClass
	 *            The class of the test.
	 * 
	 * @param testName
	 *            The name of the test case.
	 * 
	 * @param t
	 *            The failure.
	 */
	public void addFailure(Class testClass, String testName,
			AssertionFailedException t) {
		this.numFailures++;
		this.afe = t;
	}

	public void clear() {
	}

	// A method to add a description of a failure or error to the XML element
	// describing a test case.
	private void addThrowableDescription(Element element, Throwable t) {
		if (t.getMessage() != null) {
			element.setAttribute("message", t.getMessage());
		} else {
			element.setAttribute("message", t.getClass().getName());
		}
		element.setAttribute("type", t.getClass().getName());
		StackTraceElement[] ste = t.getStackTrace();
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < ste.length; i++) {
			sb.append(ste[i].toString() + "\n");
		}
		element.addContent(sb.toString());
	}

	/**
	 * A test has finished running.
	 * 
	 * @param testClass
	 *            The class of the test.
	 * 
	 * @param testName
	 *            The name of the test case.
	 * 
	 */
	public void endTest(Class testClass, String testName) {
		this.numTestsCompleted++;

		// Create an XML element describing the test case.
		Element testCaseElement = new Element("testcase");
		testCaseElement.setAttribute("classname", testClass.getName());
		testCaseElement.setAttribute("name", testName);
		double time = System.currentTimeMillis() - this.testcaseStartTime;
		time /= 1000.0;
		testCaseElement.setAttribute("time", Double.toString(time));
		this.testSuiteElement.addContent(testCaseElement);

		// If the test failed, add the appropriate bumph.
		if (this.afe != null) {
			Element e2 = new Element("failure");
			addThrowableDescription(e2, this.afe);
			testCaseElement.addContent(e2);
		}

		// If an error occurred, add the appropriate bumph.
		if (this.throwable != null) {
			Element e2 = new Element("error");
			addThrowableDescription(e2, this.throwable);
			testCaseElement.addContent(e2);
		}

		// If all tests have completed, generate the report.
		if (this.numTests == this.numTestsCompleted) {
			this.testSuiteElement.setAttribute("errors", Integer
					.toString(this.numErrors));
			this.testSuiteElement.setAttribute("failures", Integer
					.toString(this.numFailures));
			this.testSuiteElement.setAttribute("name", this.testSuiteName);
			this.testSuiteElement.setAttribute("tests", Integer
					.toString(this.numTests));
			this.testSuiteElement.setAttribute("time", Double.toString((System
					.currentTimeMillis() - this.suiteStartTime) / 1000.0));
			this.testSuiteElement.setAttribute("timestamp", (new Date())
					.toString());
			FileWriter writer;
			try {
				writer = new FileWriter(this.todir + "TEST-"
						+ this.testSuiteName + ".xml");
				XMLOutputter outputter = new XMLOutputter();
				outputter.output(this.document, writer);
				writer.close();
			} catch (IOException e) {
				throw new BuildException(e);
			}
		}
	}
}
